;;;; Copyright © 2016 Federico Beffa <beffa@fbengineering.ch>
;;;;
;;;; This library is free software; you can redistribute it and/or
;;;; modify it under the terms of the GNU Lesser General Public
;;;; License as published by the Free Software Foundation; either
;;;; version 3 of the License, or (at your option) any later version.
;;;; 
;;;; This library is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;; Lesser General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU Lesser General Public
;;;; License along with this library; if not, write to the Free Software
;;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
;;;; 

;;; Code

(define-module (restart conditions)
  ;; #:use-module ((srfi srfi-1))
  #:use-module ((restart restart-rnrs))
  ;; #:use-module ((ice-9 control))
  ;; #:use-module ((rnrs records syntactic))
  #:use-module ((rnrs exceptions))
  #:use-module ((rnrs conditions))
  #:use-module ((rnrs arithmetic fixnums))
  #:export (error:divide-by-zero
            error:divide-by-zero2

            &divide-by-zero
            divide-by-zero?
            make-divide-by-zero-condition))

(define-condition-type &divide-by-zero &violation
  make-divide-by-zero-condition
  divide-by-zero?)

(define (arithmetic-error->expression-string operator . args)
  (with-output-to-string
    (lambda()
      (write `(,operator ,@args)))))

(define (error:divide-by-zero who . irritants)
  (let* ((exp-string (apply arithmetic-error->expression-string who irritants)))
    (call-with-current-continuation
     (lambda (k)
       (raise (restartable-condition
               (('use-value
                 (format #f "Specify a value to use instead of ~A." exp-string)
                 k
                 (lambda ()
                   (display "Enter a value: ")
                   (read)))
                ('abort
                 "Abort computation."
                 (lambda () (throw 'quit))
                 (lambda () (values)))
                ('greatest-fixnum
                 (format #f "Use greatest-fixnum instead of ~A." exp-string)
                 (lambda () (k (greatest-fixnum)))
                 (lambda () (values))))
               (make-divide-by-zero-condition)
               (make-message-condition "Division by zero.")
               (make-who-condition who)
               (make-irritants-condition irritants)))))))

;;; ----------- try restart-case --------------------------

(define (error:divide-by-zero2 who . irritants)
  (let* ((exp-string (apply arithmetic-error->expression-string who irritants)))
    (restart-case
       (raise (make-restartable-condition
               (current-restarters)
               (make-divide-by-zero-condition)
               (make-message-condition "Division by zero.")
               (make-who-condition who)
               (make-irritants-condition irritants)))
       (use-value (format #f "Specify a value to use instead of ~A."
                          exp-string)
                  (value) value
                  (lambda ()
                    (display "Enter a value: ")
                    (read)))
       (abort "Abort computation."
              () without-continuation (throw 'quit))
       (greatest-fixnum (format #f "Use greatest-fixnum instead of ~A."
                                exp-string)
                        () (greatest-fixnum)))))
